/*
 * Copyright (c) 2007-2010, University of Maryland
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions
 * and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of the University of Maryland nor the names of its contributors may be used to
 * endorse or promote products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ACE Components were written in the ADAPT Project at the University of
 * Maryland Institute for Advanced Computer Study.
 */
// $Id: PersistUtil.java 132 2011-08-26 21:25:11Z toaster $
package edu.umiacs.ace.util;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.sql.DataSource;

/**
 * Utility class for accessing the entity manager stored as 'ace-amPU'.
 *
 * @author toaster
 */
public final class PersistUtil {

    /** entity manager specified in app's persistence.xml. */
    private static EntityManagerFactory emf = null;
            //= Persistence.createEntityManagerFactory("ace-amPU");
    /** db connection to aceamdb. */
    private static DataSource ds = null;

    private static ThreadLocal<EntityManager> localEntityManager = new ThreadLocal<EntityManager>();
    /**
     *
     */

    public static EntityManager getLocalManager()
    {
        if (localEntityManager.get() == null)
        {
            localEntityManager.set(getEntityManager());
        }
        return localEntityManager.get();
    }

    public static void closeLocalManager()
    {
        if (localEntityManager.get() != null)
        {
            localEntityManager.get().close();
            localEntityManager.remove();
        }
    }
    private PersistUtil() {
    }

    /** Close and set stored entitymanager to null. */
    public static void closeFactory() {
        emf.close();
        emf = null;
    }

    /**
     *
     * @return db connection located at 'java:comp/env/jdbc/aceamdb'
     */
    public static DataSource getDataSource() {
        if (ds == null) {
            try {
                Context ctx = new InitialContext();
                ds = (DataSource) ctx.lookup("java:comp/env/jdbc/aceamdb");
            } catch (NamingException e) {
                throw new RuntimeException(e);
            }
        }
        return ds;
    }

    /**
     * Persist a new  jpa entity using the embedded factory.
     *
     * @param object object to store
     */
    public static void persist(final Object object) {
        EntityManager em = getEntityManager();
        try {
            em.getTransaction().begin();
            em.persist(object);
            em.getTransaction().commit();
        } catch (Exception e) {
            java.util.logging.Logger.getLogger(PersistUtil.class.getName()).log(
                    java.util.logging.Level.SEVERE, "exception caught", e);
            em.getTransaction().rollback();
        } finally {
            em.close();
        }
    }

    /**
     * return entiry manager, will reinitailize stored factory if it has been
     * closed.
     * @return new entitymanager
     */
    public static EntityManager getEntityManager() {
        if (emf == null) {
            emf = Persistence.createEntityManagerFactory("ace-amPU");
        }
        return emf.createEntityManager();
    }
}
